import { ZodArray, ZodBoolean, ZodEnum, ZodNumber, ZodString, ZodDate, ZodObject, ZodEffects, ZodOptional } from 'zod';
import { Entity } from '../../../types';
import { FieldWithType, TypeField } from '../../orm';
type TypeMapToZod = {
    [TypeField.array]: ZodOptional<ZodArray<ZodString, 'many'>>;
    [TypeField.date]: ZodOptional<ZodDate>;
    [TypeField.number]: ZodOptional<ZodNumber>;
    [TypeField.boolean]: ZodOptional<ZodBoolean>;
    [TypeField.string]: ZodOptional<ZodString | ZodEnum<[string, ...string[]]>>;
};
type ZodShapeAttributes<E extends Entity> = Omit<{
    [K in keyof FieldWithType<E>]: TypeMapToZod[FieldWithType<E>[K]];
}, 'id'>;
export type ZodAttributesSchema<E extends Entity> = ZodEffects<ZodObject<ZodShapeAttributes<E>, 'strict'>>;
export declare const zodAttributesSchema: <E extends Entity>(fieldWithType: FieldWithType<E>) => ZodAttributesSchema<E>;
export {};
