"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecuteService = exports.isZodError = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const constants_1 = require("@nestjs/common/constants");
const core_1 = require("@nestjs/core");
const typeorm_1 = require("typeorm");
const constants_2 = require("../../../constants");
const constants_3 = require("../constants");
const helper_1 = require("../../../helper");
function isZodError(param) {
    return (param instanceof Object &&
        'message' in param &&
        Array.isArray(param.message) &&
        'path' in param.message[0]);
}
exports.isZodError = isZodError;
let ExecuteService = class ExecuteService {
    async run(params, tmpIds) {
        const queryRunner = this.dataSource.createQueryRunner();
        await queryRunner.startTransaction('READ UNCOMMITTED');
        try {
            const resultArray = await this.executeOperations(params, tmpIds);
            await queryRunner.commitTransaction();
            return resultArray;
        }
        catch (e) {
            await queryRunner.rollbackTransaction();
            throw e;
        }
        finally {
            await queryRunner.release();
        }
        return [];
    }
    async executeOperations(params, tmpIds) {
        const iterateParams = this.asyncIteratorFactory.createIterator(params, this.runOneOperation.bind(this));
        const resultArray = [];
        let i = 0;
        const tmpIdsMap = {};
        try {
            for await (const item of iterateParams) {
                const currentParams = params[i];
                const controller = this.getControllerInstance(currentParams);
                const methodName = currentParams.methodName;
                const paramsForExecute = item;
                const itemReplace = this.replaceTmpIds(paramsForExecute, tmpIdsMap);
                // @ts-ignore
                const result = await controller[methodName](...itemReplace);
                if (tmpIds[i] && result && !Array.isArray(result.data) && result.data) {
                    tmpIdsMap[tmpIds[i]] = result.data.id;
                }
                if (result instanceof Object) {
                    resultArray.push(result);
                }
                i++;
            }
        }
        catch (e) {
            this.processException(e, i);
        }
        return resultArray;
    }
    replaceTmpIds(inputParams, tmpIdsMap) {
        const bodyInput = inputParams.at(-1);
        if (!bodyInput) {
            return inputParams;
        }
        if (typeof bodyInput === 'string') {
            return inputParams;
        }
        if (typeof bodyInput === 'number') {
            return inputParams;
        }
        if (Array.isArray(bodyInput)) {
            return inputParams;
        }
        if (!('relationships' in bodyInput)) {
            return inputParams;
        }
        const { relationships } = bodyInput;
        if (!relationships) {
            return inputParams;
        }
        bodyInput.relationships = helper_1.ObjectTyped.entries(relationships).reduce((acum, [name, val]) => {
            if (Array.isArray(val)) {
                acum[name] = val.map((i) => {
                    i['id'] = tmpIdsMap[i['id']] ? tmpIdsMap[i['id']] : i['id'];
                    return i;
                });
            }
            else {
                acum[name]['id'] = tmpIdsMap[val['id']]
                    ? tmpIdsMap[val['id']]
                    : acum[name]['id'];
            }
            return acum;
        }, { ...relationships });
        inputParams[inputParams.length - 1] = bodyInput;
        return inputParams;
    }
    getControllerInstance(params) {
        const controllerClass = params.controller;
        const controllerInstanceWrapper = params.module.controllers.get(controllerClass);
        if (!controllerInstanceWrapper) {
            const error = {
                code: 'invalid_arguments',
                path: ['type'],
                message: `Controller "${controllerClass.name}" not found`,
            };
            throw new common_1.NotFoundException([error]);
        }
        return controllerInstanceWrapper.instance;
    }
    processException(e, i) {
        if (e instanceof common_1.HttpException) {
            const response = e.getResponse();
            if (isZodError(response)) {
                response['message'] = response['message'].map((m) => {
                    m['path'] = [constants_3.KEY_MAIN_INPUT_SCHEMA, `${i}`, ...m['path']];
                    return m;
                });
            }
            throw new common_1.HttpException(response, e.getStatus());
        }
        throw e;
    }
    async runOneOperation(paramForExecute) {
        const { params, controller, methodName, module } = paramForExecute;
        const pramsPipe = Object.values(Reflect.getMetadata(constants_1.ROUTE_ARGS_METADATA, controller, methodName));
        const resultParams = new Array(params.length);
        for (const { pipes, index } of pramsPipe) {
            resultParams[index] = await this.runPipes(params[index], module, pipes);
        }
        return resultParams;
    }
    async runPipes(initialParams, module, pipes) {
        let modifiedParams = initialParams;
        for (const pipe of pipes) {
            const pipeInstance = this.getPipeInstance(pipe, module);
            modifiedParams = await pipeInstance.transform(modifiedParams, {});
        }
        return modifiedParams;
    }
    getPipeInstance(pipe, module) {
        const instanceWrapper = module.getProviderByKey(pipe);
        if (!instanceWrapper) {
            return this.moduleRef.get(pipe, { strict: false });
        }
        return instanceWrapper.instance;
    }
};
exports.ExecuteService = ExecuteService;
tslib_1.__decorate([
    (0, common_1.Inject)(constants_2.CURRENT_DATA_SOURCE_TOKEN),
    tslib_1.__metadata("design:type", typeorm_1.DataSource)
], ExecuteService.prototype, "dataSource", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(core_1.ModuleRef),
    tslib_1.__metadata("design:type", core_1.ModuleRef)
], ExecuteService.prototype, "moduleRef", void 0);
tslib_1.__decorate([
    (0, common_1.Inject)(constants_3.ASYNC_ITERATOR_FACTORY),
    tslib_1.__metadata("design:type", Object)
], ExecuteService.prototype, "asyncIteratorFactory", void 0);
exports.ExecuteService = ExecuteService = tslib_1.__decorate([
    (0, common_1.Injectable)()
], ExecuteService);
//# sourceMappingURL=execute.service.js.map